@@ -4,10 +4,8 @@ use warnings;
 our %gui;
 our %prefs = (RDPCLIENT         => 'xfreerdp /size:%Xx%Y /bpp:32 +clipboard /sound /t:"%n - RemoteBox" /v:%h:%p',
               VNCCLIENT         => 'vncviewer -Shared -AcceptClipboard -SetPrimary -SendClipboard -SendPrimary -RemoteResize -DesktopSize %Xx%Y  %h:%p',
-              HEARTBEAT         => 1,
               ADDADDITIONS      => 1,
               RDPAUTOOPEN       => 1,
-              MSGLOGEXPAND      => 1,
               GUESTLISTEXPAND   => 1,
               AUTOSORTGUESTLIST => 1,
               EXTENDEDDETAILS   => 0,
@@ -40,8 +38,8 @@ sub rbprefs_get {
     $gui{THUMBDIR} = "$gui{PREFSDIR}/remoteboxthumbs";
 
     # See if we can make some more useful defaults for OSs which don't have xfreerdp
-    if ($^O =~ m/netbsd/i) { $prefs{RDPCLIENT} = 'rdesktop -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p'; }
-    elsif ($^O =~ m/solaris/i) { $prefs{RDPCLIENT} = 'rdesktop -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p'; }
+    if ($^O =~ m/netbsd/i) { $prefs{RDPCLIENT} = 'rdesktop -g %Xx%Y -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p'; }
+    elsif ($^O =~ m/solaris/i) { $prefs{RDPCLIENT} = 'rdesktop -g %Xx%Y -r sound:local -r clipboard:PRIMARYCLIPBOARD -T "%n - RemoteBox" %h:%p'; }
     elsif ($^O =~ m/MSWin/) { $prefs{RDPCLIENT} = 'mstsc /w:%X /h:%Y /v:%h:%p'; }
 
     if (open(PREFS, '<', $gui{CONFIGFILE})) {
@@ -56,7 +54,6 @@ sub rbprefs_get {
     }
     else { $prefs{URL}{qq[http://localhost:18083]} = 'URL'; } #Add a default URL
 
-    $gui{expanderMessages}->set_expanded($prefs{MSGLOGEXPAND});
     $gui{checkbuttonShowDetails}->set_active($prefs{EXTENDEDDETAILS});
 
     # Load in the connection profiles
@@ -79,48 +76,44 @@ sub rbprefs_save {
     mkdir($gui{PREFSDIR}, 0755) unless (-e $gui{PREFSDIR});
 
     if (open(PREFS, '>', $gui{CONFIGFILE})) {
-        foreach my $key (keys %prefs) {
+        foreach my $key (sort(keys %prefs)) {
             if ($key eq 'URL') {
                 foreach (keys %{$prefs{URL}}) { print PREFS "URL=$_\n"; }
             }
             elsif ($key eq 'USER') {
                 foreach (keys %{$prefs{USER}}) { print PREFS "USER=$_\n"; }
             }
-            else { print(PREFS "$key=$prefs{$key}\n"); }
+            else { print PREFS "$key=$prefs{$key}\n"; }
         }
         close(PREFS);
     }
     else { warn "Unable to save preferences: $gui{CONFIGFILE}\n"; }
-
-
 }
 
-# Saves the profiles.
-# DO NOT call this from rbprefs_save otherwise there will be a race condition which will
-# empty the profiles file.
+# Saves the profiles.DO NOT call this from rbprefs_save otherwise there will be
+# a race condition which will empty the profiles file.
 sub rbprofiles_save {
     # As this file might contain passwords, we'll create it 0600, but only if it's the first time we're creating it.
-    my $new = 1;
-    $new = 0 if (-e $gui{PROFILESFILE});
+    my $newprofilesfile = (-e $gui{PROFILESFILE}) ? 0 : 1;
 
     # Save the connection profiles
     if (open(PROFILES, '>', $gui{PROFILESFILE})) {
-        chmod(0600, $gui{PROFILESFILE}) if ($new);
+        chmod(0600, $gui{PROFILESFILE}) if ($newprofilesfile);
         my $iter = $gui{liststoreProfiles}->get_iter_first();
 
-        while($iter) {
-            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 0) . "\a");
-            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 1) . "\a");
-            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 2) . "\a");
+        while(defined($iter) and $gui{liststoreProfiles}->iter_is_valid($iter)) {
+            print(PROFILES $gui{liststoreProfiles}->get_value($iter, 0) . "\a" .
+                           $gui{liststoreProfiles}->get_value($iter, 1) . "\a" .
+                           $gui{liststoreProfiles}->get_value($iter, 2) . "\a");
             my $password = $gui{liststoreProfiles}->get_value($iter, 3);
             my $key = &random_key(length($password));
             $password = &xor_pass($password, $key);
             my @ordpass = unpack("C*", $password);
             $password = join(' ', @ordpass);
-            print(PROFILES "$password\a");
-            print(PROFILES "$key\n");
-            $iter = $gui{liststoreProfiles}->iter_next($iter);
+            print PROFILES "$password\a$key\n";
+            $gui{liststoreProfiles}->iter_next($iter); # The iter is now modified directly
         }
+
         close(PROFILES);
     }
 }
@@ -128,9 +121,8 @@ sub rbprofiles_save {
 # Save RemoteBox's window position for restoration later
 sub save_window_pos {
     my ($winname) = @_;
-    my $alloc = $gui{$winname}->allocation;
-    my ($w, $h) = ($alloc->width, $alloc->height);
-    my ($x,$y) = $gui{$winname}->get_position();
+    my ($w, $h) = $gui{$winname}->get_size();
+    my ($x, $y) = $gui{$winname}->get_position();
     $prefs{"WINPOS_$winname"} = "$w:$h:$x:$y";
     &rbprefs_save();
 }
@@ -164,12 +156,25 @@ sub show_dialog_editnat {
         }
         else { $gui{dialogNATDetails}->hide; }
 
-    } until (!$gui{dialogNATDetails}->visible());
+    } until (!$gui{dialogNATDetails}->get_visible());
 }
 
 # Show dialog for adding an IPv4 port forwarding rule
 sub show_dialog_pf4 {
-    $gui{entryPFIPv4Name}->set_text('Rule ' . int(rand(9999)));
+    my ($widget, $mode) = @_;
+    my $oldname;
+
+    if ($mode eq 'EDIT') {
+        my $rule = &getsel_list_pf4();
+        $oldname = $$rule{Name};
+        $gui{entryPFIPv4Name}->set_text($$rule{Name});
+        &combobox_set_active_text($gui{comboboxPFIPv4Protocol}, $$rule{Protocol}, 1);
+        $gui{entryPFIPv4HostIP}->set_text($$rule{HostIP});
+        $gui{spinbuttonPFIPv4HostPort}->set_value($$rule{HostPort});
+        $gui{entryPFIPv4GuestIP}->set_text($$rule{GuestIP});
+        $gui{spinbuttonPFIPv4GuestPort}->set_value($$rule{GuestPort});
+    }
+    else { $gui{entryPFIPv4Name}->set_text('Rule ' . int(rand(999999))); }
 
     do {
         my $response = $gui{dialogPFIPv4}->run;
@@ -182,25 +187,39 @@ sub show_dialog_pf4 {
             else {
                 $gui{dialogPFIPv4}->hide;
                 my $natref = &getsel_list_vbprefsnat();
+                # We can't actually edit an existing rule, so it's deleted and recreated.
+                INATNetwork_removePortForwardRule($$natref{INATNetwork}, 0, $oldname) if ($mode eq 'EDIT');
                 INATNetwork_addPortForwardRule($$natref{INATNetwork}, 0,
-                                       $gui{entryPFIPv4Name}->get_text(),
-                                       &getsel_combo($gui{comboboxPFIPv4Protocol}, 1),
-                                       $gui{entryPFIPv4HostIP}->get_text(),
-                                       int($gui{adjustmentPFIPHostPort}->get_value()),
-                                       $gui{entryPFIPv4GuestIP}->get_text(),
-                                       int($gui{adjustmentPFIPGuestPort}->get_value()));
-
+                                               $gui{entryPFIPv4Name}->get_text(),
+                                               &getsel_combo($gui{comboboxPFIPv4Protocol}, 1),
+                                               $gui{entryPFIPv4HostIP}->get_text(),
+                                               int($gui{adjPFIPHostPort}->get_value()),
+                                               $gui{entryPFIPv4GuestIP}->get_text(),
+                                               int($gui{adjPFIPGuestPort}->get_value()));
                 &fill_list_pf4($$natref{INATNetwork});
             }
         }
         else { $gui{dialogPFIPv4}->hide; }
 
-    } until (!$gui{dialogPFIPv4}->visible());
+    } until (!$gui{dialogPFIPv4}->get_visible());
 }
 
 # Show dialog for adding an IPv6 port forwarding rule
 sub show_dialog_pf6 {
-    $gui{entryPFIPv6Name}->set_text('Rule ' . int(rand(9999)));
+    my ($widget, $mode) = @_;
+    my $oldname;
+
+    if ($mode eq 'EDIT') {
+        my $rule = &getsel_list_pf6();
+        $oldname = $$rule{Name};
+        $gui{entryPFIPv6Name}->set_text($$rule{Name});
+        &combobox_set_active_text($gui{comboboxPFIPv6Protocol}, $$rule{Protocol}, 1);
+        $gui{entryPFIPv6HostIP}->set_text($$rule{HostIP});
+        $gui{spinbuttonPFIPv6HostPort}->set_value($$rule{HostPort});
+        $gui{entryPFIPv6GuestIP}->set_text($$rule{GuestIP});
+        $gui{spinbuttonPFIPv6GuestPort}->set_value($$rule{GuestPort});
+    }
+    else { $gui{entryPFIPv6Name}->set_text('Rule ' . int(rand(999999))); }
 
     do {
         my $response = $gui{dialogPFIPv6}->run;
@@ -213,20 +232,21 @@ sub show_dialog_pf6 {
             else {
                 $gui{dialogPFIPv6}->hide;
                 my $natref = &getsel_list_vbprefsnat();
+                # We can't actually edit an existing rule, so it's deleted and recreated.
+                INATNetwork_removePortForwardRule($$natref{INATNetwork}, 1, $oldname) if ($mode eq 'EDIT');
                 INATNetwork_addPortForwardRule($$natref{INATNetwork}, 1,
-                                       $gui{entryPFIPv6Name}->get_text(),
-                                       &getsel_combo($gui{comboboxPFIPv6Protocol}, 1),
-                                       $gui{entryPFIPv6HostIP}->get_text(),
-                                       int($gui{adjustmentPFIPHostPort}->get_value()),
-                                       $gui{entryPFIPv6GuestIP}->get_text(),
-                                       int($gui{adjustmentPFIPGuestPort}->get_value()));
-
+                                               $gui{entryPFIPv6Name}->get_text(),
+                                               &getsel_combo($gui{comboboxPFIPv6Protocol}, 1),
+                                               $gui{entryPFIPv6HostIP}->get_text(),
+                                               int($gui{adjPFIPHostPort}->get_value()),
+                                               $gui{entryPFIPv6GuestIP}->get_text(),
+                                               int($gui{adjPFIPGuestPort}->get_value()));
                 &fill_list_pf6($$natref{INATNetwork});
             }
         }
         else { $gui{dialogPFIPv6}->hide; }
 
-    } until (!$gui{dialogPFIPv6}->visible());
+    } until (!$gui{dialogPFIPv6}->get_visible());
 }
 
 # Removes an IPv4 port forwarding rule to the selected NAT Network
@@ -248,7 +268,6 @@ sub show_dialog_prefs {
     $gui{entryPrefsRDPClient}->set_text($prefs{RDPCLIENT});
     $gui{entryPrefsVNCClient}->set_text($prefs{VNCCLIENT});
     $gui{checkbuttonPrefsRDPAuto}->set_active($prefs{RDPAUTOOPEN});
-    $gui{checkbuttonPrefsHeartbeat}->set_active($prefs{HEARTBEAT});
     $gui{checkbuttonPrefsAddAdditions}->set_active($prefs{ADDADDITIONS});
     $gui{checkbuttonPrefsAutoSortGuestList}->set_active($prefs{AUTOSORTGUESTLIST});
     $gui{checkbuttonPrefsAutoHint}->set_active($prefs{AUTOHINTDISP});
@@ -270,7 +289,6 @@ sub show_dialog_prefs {
         $prefs{RDPCLIENT} = $gui{entryPrefsRDPClient}->get_text();
         $prefs{VNCCLIENT} = $gui{entryPrefsVNCClient}->get_text();
         $prefs{RDPAUTOOPEN} = int($gui{checkbuttonPrefsRDPAuto}->get_active());
-        $prefs{HEARTBEAT} = int($gui{checkbuttonPrefsHeartbeat}->get_active());
         $prefs{ADDADDITIONS} = int($gui{checkbuttonPrefsAddAdditions}->get_active());
         $prefs{AUTOSORTGUESTLIST} = int($gui{checkbuttonPrefsAutoSortGuestList}->get_active());
         $prefs{STOPTYPE} = &getsel_combo($gui{comboboxPrefsDefaultStop}, 1);
@@ -374,7 +392,7 @@ sub show_dialog_edithon {
         }
         else { $gui{dialogHON}->hide(); }
 
-    } until (!$gui{dialogHON}->visible());
+    } until (!$gui{dialogHON}->get_visible());
 }
 
 sub vbprefs_reset {
@@ -394,12 +412,12 @@ sub vbprefs_reset {
 sub vbprefs_createhon {
     my $IHost = IVirtualBox_getHost($gui{websn});
     my ($hostinterface, $IProgress) = IHost_createHostOnlyNetworkInterface($IHost);
-    &show_progress_window($IProgress, 'Adding host only network', $gui{img}{ProgressNetwork});
+    &show_progress_window($IProgress, 'Adding host only network', $gui{pb}{ProgressNetwork});
     &fill_list_vbprefshon();
 }
 
 sub vbprefs_createnat {
-    IVirtualBox_createNATNetwork($gui{websn}, 'NatNetwork' . int(rand(9999)));
+    IVirtualBox_createNATNetwork($gui{websn}, 'NatNetwork ' . int(rand(999999)));
     &fill_list_vbprefsnat();
 }
 
@@ -417,7 +435,7 @@ sub vbprefs_removehon {
 
     my $IHost = IVirtualBox_getHost($gui{websn});
     my $IProgress = IHost_removeHostOnlyNetworkInterface($IHost, $$ifref{Uuid});
-    &show_progress_window($IProgress, 'Removing host only network');
+    &show_progress_window($IProgress, 'Removing host only network', $gui{pb}{ProgressNetwork});
     &fill_list_vbprefshon();
 }
 
@@ -429,20 +447,14 @@ sub vbprefs_removenat {
 
 # Can't use getsel_list_vbprefsnat in here due to the way the signals are propagated
 sub vbprefs_togglenat {
-    my ($widget, $path_str, $model) = @_;
-    my $iter = $model->get_iter(Gtk2::TreePath->new_from_string($path_str));
-    my $val = $model->get($iter, 0);
-    my $INATNetwork = $model->get($iter, 2);
+    my ($widget, $path_str) = @_;
+    my $iter = $gui{liststoreVBPrefsNAT}->get_iter(Gtk3::TreePath->new_from_string($path_str));
+    my $val = $gui{liststoreVBPrefsNAT}->get($iter, 0);
+    my $INATNetwork = $gui{liststoreVBPrefsNAT}->get($iter, 2);
     INATNetwork_setEnabled($INATNetwork, !$val);
     &fill_list_vbprefsnat();
 }
 
-sub rbprefs_msglog {
-    my ($widget) = @_;
-    $prefs{MSGLOGEXPAND} = int($widget->get_expanded()); # int forces 0 on undefined
-    &rbprefs_save();
-}
-
 sub rbprefs_sslverify {
     my $active = $gui{checkbuttonConnectSSL}->get_active();
     $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = $active;
@@ -450,19 +462,6 @@ sub rbprefs_sslverify {
     &rbprefs_save();
 }
 
-sub rbprefs_autohint {
-    if ($gui{checkbuttonPrefsAutoHint}->get_active()) {
-        $gui{spinbuttonPrefsWidth}->set_sensitive(1);
-        $gui{spinbuttonPrefsHeight}->set_sensitive(1);
-        $gui{comboboxPrefsDepth}->set_sensitive(1);
-    }
-    else {
-        $gui{spinbuttonPrefsWidth}->set_sensitive(0);
-        $gui{spinbuttonPrefsHeight}->set_sensitive(0);
-        $gui{comboboxPrefsDepth}->set_sensitive(0);
-    }
-}
-
 sub vbprefs_hon_dhcp_toggle {
     $gui{tableHONDHCP}->set_sensitive($gui{checkbuttonHONDHCP}->get_active()); # Ghost/Unghost other widgets based on dhcp enabled
 }
